//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property  indicator_separate_window
#property  indicator_buffers 3
#property  indicator_color1  LimeGreen
#property  indicator_color2  Gold
#property  indicator_color3  Silver
#property  indicator_style3  STYLE_DOT
#property  indicator_minimum 0

extern int    periods  = 20;
extern double treshold = 20;
extern int    MaPeriod = 14;
extern int    MaMethod = MODE_EMA;
extern bool   AdaptiveTreshold   = false;
extern int    AtrPeriod          = 14;
extern double AtrFactor          = 1.5;
extern bool   alertsOn           = false;
extern bool   alertsOnCurrent    = true;
extern bool   alertsOnLevelCross = true;
extern bool   alertsOnMaCross    = true;
extern bool   alertsMessage      = true;
extern bool   alertsPushNotif    = false;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;

double width[];
double ma[];
double trend1[];
double trend2[];
double tresh[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,width); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,ma);
   SetIndexBuffer(2,tresh);
   SetIndexBuffer(3,trend1);   
   SetIndexBuffer(4,trend2);
   IndicatorShortName("Donchian Channel width("+periods+")");
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         double pipMultiplier = MathPow(10,Digits%2);
         double pip           = Point*pipMultiplier;

   //
   //
   //
   //
   //
   //

   for(int i=limit; i >= 0; i--)
   {
      double upper = iHigh(Symbol(),Period(),iHighest(Symbol(),Period(),MODE_HIGH,periods,i));
      double lower = iLow(Symbol(),Period(),iLowest(Symbol(),Period(),MODE_LOW,periods,i));
         width[i] = (upper-lower)/pip;
   }
   for(i=limit; i >= 0; i--)
   {
      ma[i] = iMAOnArray(width,0,MaPeriod,0,MaMethod,i);
      trend1[i] = trend1[i+1];
      trend2[i] = trend2[i+1];
      if (AdaptiveTreshold)
         treshold = iATR(NULL,0,AtrPeriod,i)*AtrFactor/pip;
         tresh[i] = treshold;
         if (width[i]>treshold) trend1[i] =  1;
         if (width[i]<treshold) trend1[i] = -1;
         if (width[i]>ma[i])    trend2[i] =  1;
         if (width[i]<ma[i])    trend2[i] = -1;
   }   
   manageAlerts();
   return(0);
}
  
//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      
      //
      //
      //
      //
      //
      
      static string   alertType1 = "";
      static datetime alertTime1 = 0;
      if (alertsOnLevelCross && trend1[whichBar] != trend1[whichBar+1])
      {
         if (trend1[whichBar] ==  1)  doAlert(alertType1,alertTime1,"crossed "+DoubleToStr(treshold,2)  +" up");
         if (trend1[whichBar] == -1)  doAlert(alertType1,alertTime1,"crossed "+DoubleToStr(treshold,2)  +" down");
      }
      static string   alertType2 = "";
      static datetime alertTime2 = 0;
      if (alertsOnMaCross && trend2[whichBar] != trend2[whichBar+1])
      {
         if (trend2[whichBar] ==  1) doAlert(alertType2,alertTime2,"crossed average up");
         if (trend2[whichBar] == -1) doAlert(alertType2,alertTime2,"crossed average down");
      }
   }
}

//
//
//
//
//

void doAlert(string& previousAlert, datetime& previousTime, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," donchian width ",doWhat);
          if (alertsMessage)   Alert(message);
          if (alertsEmail)     SendMail(StringConcatenate(Symbol(),"donchian width "),message);
          if (alertsPushNotif) SendNotification(StringConcatenate(Symbol(),"donchian width "+" "+message));
          if (alertsSound)     PlaySound("alert2.wav");
   }
}
  